@@ -1,4 +1,5 @@
 from .classes import *
+from .settings import lonlat_precision, xy_precision
 
 
 def consolidateComplexIntersections(network):
@@ -18,15 +19,19 @@ def consolidateComplexIntersections(network):
         new_node.node_id = network.max_node_id
         new_node.main_node_id = main_node_id
         new_node.ctrl_type = 1
-        x_coord_sum = 0
-        y_coord_sum = 0
+        osm_node_id_list = []
+        x_coord_sum, y_coord_sum = 0.0, 0.0
+        x_coord_xy_sum, y_coord_xy_sum = 0.0, 0.0
 
         for node in node_group:
             node.valid = False
             removal_node_set.add(node)
 
+            osm_node_id_list.append(node.osm_node_id)
             x_coord_sum += node.geometry.x
             y_coord_sum += node.geometry.y
+            x_coord_xy_sum += node.geometry_xy.x
+            y_coord_xy_sum += node.geometry_xy.y
 
             for link in node.incoming_link_list:
                 if link.from_node in node_group:
@@ -45,7 +50,13 @@ def consolidateComplexIntersections(network):
 
             new_node.osm_highway = node.osm_highway
 
-        new_node.geometry = geometry.Point(x_coord_sum / len(node_group), y_coord_sum / len(node_group))
+        new_node.osm_node_id = ';'.join(osm_node_id_list)
+        x_coord_ave = round(x_coord_sum / len(node_group),lonlat_precision)
+        y_coord_ave = round(y_coord_sum / len(node_group),lonlat_precision)
+        new_node.geometry = geometry.Point(x_coord_ave, y_coord_ave)
+        x_coord_xy_ave = round(x_coord_xy_sum / len(node_group),xy_precision)
+        y_coord_xy_ave = round(y_coord_xy_sum / len(node_group),xy_precision)
+        new_node.geometry_xy = geometry.Point(x_coord_xy_ave, y_coord_xy_ave)
 
         network.node_dict[new_node.node_id] = new_node
         network.max_node_id += 1
