@@ -3,10 +3,16 @@ from .combine_links import *
 from .complex_intersection import *
 from .wayfilters import *
 from .pois import *
+from .coordconvertor import from_latlon
 import re
 from shapely import wkt
+import numpy as np
 
 
+# todo: highway type britain-and-ireland
+# todo: node name
+# todo: dir_flag
+
 def _newLinkFromWay(link_id, way, direction, ref_node_list):
     link = Link()
     link.osm_way_id = way.osm_way_id
@@ -41,7 +47,7 @@ def _newLinkFromWay(link_id, way, direction, ref_node_list):
     link.to_node = ref_node_list[-1]
     link.from_node.outgoing_link_list.append(link)
     link.to_node.incoming_link_list.append(link)
-    link.geometry = getLineFromRefNodes(ref_node_list)
+    link.geometry, link.geometry_xy = getLineFromRefNodes(ref_node_list)
     link.calculateLength()
     return link
 
@@ -51,8 +57,11 @@ def _createNodeOnBoundary(node_in,node_outside,network):
     node.node_id = network.max_node_id
     line = network.bounds.intersection(geometry.LineString([node_in.geometry,node_outside.geometry]))
     lon, lat = line.coords[1]
-    node.geometry = geometry.Point((round(lon,7),round(lat,7)))
+    node.geometry = geometry.Point((round(lon,lonlat_precision),round(lat,lonlat_precision)))
+    x, y = from_latlon(lon, lat, network.central_lon)
+    node.geometry_xy = geometry.Point((round(x,xy_precision),round(y,xy_precision)))
     node.is_crossing = True
+    node.notes = 'boundary node created by osm2gmns'
     network.node_dict[node.node_id] = node
     network.max_node_id += 1
     return node
@@ -246,38 +255,57 @@ def _updateDefaultLaneSpeed(default_lanes, default_speed, network):
 
 def _parseNodes(network, nodes, strict_mode):
     osm_node_dict = {}
+    osm_node_id_list = []
+    osm_node_coord_list = []
+
     for osm_node in nodes:
         node = Node()
         node.osm_node_id = str(osm_node.id)
         lon,lat = osm_node.lonlat
-        node.geometry = geometry.Point((round(lon,7),round(lat,7)))
-        # node.geometry = geometry.Point(osm_node.lonlat)
+        node.geometry = geometry.Point((round(lon,lonlat_precision),round(lat,lonlat_precision)))
 
         if strict_mode:
             if not node.geometry.within(network.bounds):
                 node.in_region = False
 
+        osm_node_id_list.append(node.osm_node_id)
+        osm_node_coord_list.append((lon,lat))
+
         tags = osm_node.tags
         if 'highway' in tags.keys():
             node.osm_highway = tags['highway']
         if 'signal' in node.osm_highway: node.ctrl_type = 1         # todo: check signalized tag
 
         osm_node_dict[node.osm_node_id] = node
+
+    coord_array = np.array(osm_node_coord_list)
+    central_lon = coord_array[:,0].mean()
+    network.central_lon = central_lon
+    xs, ys = from_latlon(coord_array[:,0], coord_array[:,1], central_lon)
+    for node_no, node_id in enumerate(osm_node_id_list):
+        node = osm_node_dict[node_id]
+        node.geometry_xy = geometry.Point((round(xs[node_no],xy_precision),round(ys[node_no],xy_precision)))
+
     network.osm_node_dict = osm_node_dict
 
 
 def _parseWays(network, ways, relations, network_type, POIs):
     osm_way_dict = {}
+    osm_node_dict = network.osm_node_dict
 
     for osm_way in ways:
         way = Way()
         way.osm_way_id = str(osm_way.id)
 
+        valid = True
         for ref_node_id in osm_way.refs:
             try:
-                way.ref_node_list.append(network.osm_node_dict[str(ref_node_id)])
+                way.ref_node_list.append(osm_node_dict[str(ref_node_id)])
             except KeyError:
-                print(f'  warning: ref node {ref_node_id} in way {way.osm_way_id} is not defined')
+                valid = False
+                print(f'  warning: ref node {ref_node_id} in way {way.osm_way_id} is not defined, way {way.osm_way_id} will not be imported')
+        if not valid:
+            continue
 
         tags = osm_way.tags
         if 'highway' in tags.keys():
@@ -323,7 +351,7 @@ def _parseWays(network, ways, relations, network_type, POIs):
                 # todo: reversible, alternating: https://wiki.openstreetmap.org/wiki/Tag:oneway%3Dreversible
                 way.oneway = False
             else:
-                printlog(f'new maxspeed type detected at way {way.osm_way_id}, {tags["oneway"]}', 'warning')
+                printlog(f'new lane type detected at way {way.osm_way_id}, {tags["oneway"]}', 'warning')
         if 'area' in tags.keys():
             way.area = tags['area']
         if 'motor_vehicle' in tags.keys():
@@ -352,13 +380,23 @@ def _parseWays(network, ways, relations, network_type, POIs):
     include_aeroway = True if 'aeroway' in network_type_set else False
 
     for osm_way_id, way in osm_way_dict.items():
-        if way.highway:
+        if way.building or way.amenity:
+            POI_way_list.append(way)
+        elif way.highway:
+            if way.highway in highway_poi_set:
+                way.way_poi = way.highway
+                POI_way_list.append(way)
+                continue
+            if way.area and way.area != 'no':
+                continue
+            if way.highway in negligible_highway_type_set:
+                continue
+
             try:
                 way.link_type_name = osm_highway_type_dict[way.highway]
                 way.link_type = link_type_no_dict[way.link_type_name]
             except KeyError:
-                if way.highway not in negligible_link_type_list:
-                    printlog(f'new highway type at way {way.osm_way_id}, {way.highway}', 'warning')
+                printlog(f'new highway type at way {way.osm_way_id}, {way.highway}', 'warning')
                 continue
 
             if len(way.ref_node_list) < 2: continue
@@ -380,30 +418,49 @@ def _parseWays(network, ways, relations, network_type, POIs):
                 if network.default_speed:
                     way.maxspeed = network.default_speed[way.link_type_name]
             link_way_list.append(way)
+            if way.lanes is None: network.complete_highway_lanes = False
 
         elif way.railway:
             if not include_railway: continue
-            if way.area: continue
+
+            if way.railway in railway_poi_set:
+                way.way_poi = way.railway
+                POI_way_list.append(way)
+                continue
+            if way.area and way.area != 'no':
+                continue
+            if way.railway in negligible_railway_type_set:
+                continue
+
             way.link_type_name = way.railway
             way.link_type = link_type_no_dict['railway']
             if way.oneway is None:
                 way.oneway = default_oneway_flag_dict['railway']
             link_way_list.append(way)
+
         elif way.aeroway:
             if not include_aeroway: continue
-            if way.area: continue
+
+            if way.aeroway in aeroway_poi_set:
+                way.way_poi = way.aeroway
+                POI_way_list.append(way)
+                continue
+            if way.area and way.area != 'no':
+                continue
+            if way.aeroway in negligible_aeroway_type_set:
+                continue
+
             way.link_type_name = way.aeroway
             way.link_type = link_type_no_dict['aeroway']
             if way.oneway is None:
                 way.oneway = default_oneway_flag_dict['aeroway']
             link_way_list.append(way)
-        elif way.building or way.amenity:
-            POI_way_list.append(way)
+
         else:
             pass
 
     _identifyCrossingNodes(link_way_list)
-    _getNetworkNodes(network)
+    _getNetworkNodes(network)               # osm node -> node
 
     _identifyPureCycleWays(link_way_list)
     _createLinks(network, link_way_list)
@@ -418,8 +475,9 @@ def _parseOSM(network, nodes, ways, relations, strict_mode, network_type, POIs):
     _parseWays(network, ways, relations, network_type, POIs)
 
 
-def _buildNet(netdata,network_type, POIs,strict_mode, min_nodes, combine, int_buffer, bbox, default_lanes, default_speed):
+def _buildNet(netdata,network_type_valid, POIs, strict_mode, min_nodes, combine, int_buffer, bbox, default_lanes, default_speed):
     network = Network()
+
     _updateDefaultLaneSpeed(default_lanes, default_speed, network)
 
     bounds, nodes, ways, relations = netdata['bounds'], netdata['nodes'], netdata['ways'], netdata['relations']
@@ -427,7 +485,6 @@ def _buildNet(netdata,network_type, POIs,strict_mode, min_nodes, combine, int_bu
     minlat, minlon, maxlat, maxlon = bbox if bbox else bounds['minlat'], bounds['minlon'], bounds['maxlat'], bounds['maxlon']
     network.bounds = geometry.Polygon([(minlon, maxlat), (maxlon, maxlat), (maxlon, minlat), (minlon, minlat)])
 
-    network_type_valid = _getValidNetworkType(network_type)
     _parseOSM(network, nodes, ways, relations, strict_mode, network_type_valid, POIs)
 
     # remove isolated nodes and links
@@ -442,6 +499,18 @@ def _buildNet(netdata,network_type, POIs,strict_mode, min_nodes, combine, int_bu
     return network
 
 
+def _getNetClassDict(network_type_valid):
+    # was used to reduce memory usage
+    net_class_dict = {'highway':False, 'railway':False, 'aeroway':False}
+    if 'auto' in network_type_valid or 'bike' in network_type_valid or 'walk' in network_type_valid:
+        net_class_dict['highway'] = True
+    if 'railway' in network_type_valid:
+        net_class_dict['railway'] = True
+    if 'aeroway' in network_type_valid:
+        net_class_dict['aeroway'] = True
+    return net_class_dict
+
+
 def getNetFromOSMFile(osm_filename='map.osm', network_type=('auto',), POIs=False, strict_mode=True, min_nodes=1,
                       combine=False, int_buffer=default_int_buffer, bbox=None, default_lanes=False, default_speed=False):
     """
@@ -458,8 +527,9 @@ def getNetFromOSMFile(osm_filename='map.osm', network_type=('auto',), POIs=False
     :return: a network instance
     """
 
+    network_type_valid = _getValidNetworkType(network_type)
     netdata = readXMLFile(osm_filename)
-    network = _buildNet(netdata,network_type, POIs,strict_mode, min_nodes, combine, int_buffer, bbox, default_lanes, default_speed)
+    network = _buildNet(netdata,network_type_valid, POIs,strict_mode, min_nodes, combine, int_buffer, bbox, default_lanes, default_speed)
     return network
 
 
@@ -478,55 +548,80 @@ def getNetFromPBFFile(pbf_filename='map.osm.pbf', network_type=('auto',), POIs=F
     :param default_speed: True; False; Dict
     :return: a network instance
     """
+
+    network_type_valid = _getValidNetworkType(network_type)
     netdata = readPBFFile(pbf_filename)
-    network = _buildNet(netdata,network_type, POIs,strict_mode, min_nodes, combine, int_buffer, bbox, default_lanes, default_speed)
+    network = _buildNet(netdata,network_type_valid, POIs,strict_mode, min_nodes, combine, int_buffer, bbox, default_lanes, default_speed)
     return network
 
 
-def getNetFromCSV(folder=''):
-    node_data, link_data = readCSVFile(folder)
+def getNetFromCSV(folder='', enconding=None):
+    node_data, link_data = readCSVFile(folder, enconding)
     network = Network()
     max_node_id = 0
     max_link_id = 0
+    node_id_list = []
+    node_coord_list = []
 
-    for i in range(len(node_data)):
+    for node_info in node_data:
         node = Node()
-        node.node_id = node_data.loc[i, 'node_id']
+        node.name = node_info['name']
+        node.node_id = int(node_info['node_id'])
         if node.node_id > max_node_id: max_node_id = node.node_id
-        osm_node_id = node_data.loc[i, 'osm_node_id']
-        if osm_node_id == osm_node_id: node.osm_node_id = osm_node_id
-        osm_highway = node_data.loc[i, 'osm_highway']
-        if osm_highway == osm_highway: node.osm_highway = osm_highway
-        x_coord, y_coord = node_data.loc[i, 'x_coord'], node_data.loc[i, 'y_coord']
-        node.geometry = geometry.Point(x_coord,y_coord)
-        main_node_id = node_data.loc[i, 'main_node_id']
-        if main_node_id == main_node_id: node.main_node_id = int(main_node_id)
-        node.ctrl_type = node_data.loc[i, 'ctrl_type']
-        poi_id = node_data.loc[i, 'poi_id']
-        if poi_id == poi_id: node.poi_id = int(poi_id)
+        node.osm_node_id = node_info['osm_node_id']
+        node.osm_highway = node_info['osm_highway']
+        x_coord, y_coord = float(node_info['x_coord']), float(node_info['y_coord'])
+        node.geometry = geometry.Point(x_coord, y_coord)
+
+        node_id_list.append(node.node_id)
+        node_coord_list.append((x_coord, y_coord))
+
+        main_node_id = node_info['main_node_id']
+        if main_node_id: node.main_node_id = main_node_id
+        node.ctrl_type = node_info['ctrl_type']
+        poi_id = node_info['poi_id']
+        if poi_id: node.poi_id = poi_id
+
         network.node_dict[node.node_id] = node
     network.max_node_id = max_node_id + 1
 
-    for i in range(len(link_data)):
+    coord_array = np.array(node_coord_list)
+    central_lon = coord_array[:,0].mean()
+    network.central_lon = central_lon
+    xs, ys = from_latlon(coord_array[:,0], coord_array[:,1], central_lon)
+    for node_no, node_id in enumerate(node_id_list):
+        node = network.node_dict[node_id]
+        node.geometry_xy = geometry.Point((round(xs[node_no],xy_precision),round(ys[node_no],xy_precision)))
+
+
+    for link_info in link_data:
         link = Link()
-        name = link_data.loc[i, 'name']
-        if name == name: link.name = name
-        link.link_id = link_data.loc[i, 'link_id']
-        if link.link_id > max_link_id: max_link_id = link.link_id
-        link.osm_way_id = link_data.loc[i, 'osm_way_id']
-        link.from_node = network.node_dict[link_data.loc[i, 'from_node_id']]
-        link.to_node = network.node_dict[link_data.loc[i, 'to_node_id']]
-        link.length = link_data.loc[i, 'length']
-        lanes = link_data.loc[i, 'lanes']
-        if lanes == lanes: link.lanes = lanes
-        free_speed = link_data.loc[i, 'free_speed']
-        if free_speed == free_speed: link.free_speed = free_speed
-        link.link_type_name = link_data.loc[i, 'link_type_name']
-        link.link_type = link_data.loc[i, 'link_type']
-        link.geometry = wkt.loads(link_data.loc[i, 'geometry'])
-        from_biway = link_data.loc[i, 'from_biway']
-        if from_biway == from_biway:
-            if int(from_biway) == 1: link.from_bidirectional_way = True
+        link.name = link_info['name']
+        link.link_id = int(link_info['link_id'])
+        link.osm_way_id = link_info['osm_way_id']
+        link.from_node = network.node_dict[int(link_info['from_node_id'])]
+        link.to_node = network.node_dict[int(link_info['to_node_id'])]
+        link.length = float(link_info['length'])
+        lanes = link_info['lanes']
+        if lanes:
+            link.lanes = int(lanes)
+        else:
+            network.complete_highway_lanes = False
+        free_speed = link_info['free_speed']
+        if free_speed: link.free_speed = float(free_speed)
+        link.link_type_name = link_info['link_type_name']
+        link.link_type = int(link_info['link_type'])
+
+        link.geometry = wkt.loads(link_info['geometry'])
+        coord_array = np.array(link.geometry.coords)
+        xs, ys = from_latlon(coord_array[:, 0], coord_array[:, 1], central_lon)
+        xys = np.vstack((xs, ys)).T
+        link.geometry_xy = geometry.LineString(xys)
+
+        link.allowed_uses = link_info['allowed_uses']
+        from_biway = int(link_info['from_biway'])
+        if from_biway == 1:
+            link.from_bidirectional_way = True
 
         network.link_dict[link.link_id] = link
         link.from_node.outgoing_link_list.append(link)
